from typing import Any, Callable, Optional

from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated

from harambe.normalize_url import normalize_url
from harambe.types import URL


class ParserTypeUrl:
    def __new__(cls, base_url: Optional[URL] = None) -> Any:
        return Annotated[str, AfterValidator(cls.validate_type(base_url))]

    @staticmethod
    def validate_type(base_url: Optional[URL]) -> Callable[[URL], str]:
        def _validate_type(url: URL) -> str:
            return normalize_url(url, base_url)

        return _validate_type



from harambe.types import ScrapeResult, Schema


class HarambeException(Exception):
    """Base exception for all custom exceptions in Harambe."""

    pass


class SchemaValidationError(HarambeException):
    def __init__(self, schema: Schema, data: ScrapeResult, message: str):
        super().__init__(f"Data {data} does not match schema {schema}. {message}")


class CaptchaError(HarambeException):
    def __init__(self, message: str = "CAPTCHA was hit.") -> None:
        super().__init__(message)
